#!/usr/bin/env python3

from pequod_agent.cluster.controllers import Controller, BootstrapController, ClusterController
from pequod_agent.cluster.manager import CoreManager, BootstrapCore, ClusterCore
from pequod_agent.cluster.lbupdater import LBUpdater, BootstrapLBUpdater, ClusterLBUpdater
from pequod_agent.cluster.registries import Registry, BootstrapRegistry, ClusterRegistry
from pequod_agent.utilities.metaclasses import Singleton


class Cluster(metaclass=Singleton):
    def __init__(self,
                 pequod_controller: Controller,
                 core_manager: CoreManager,
                 lb_updater: LBUpdater,
                 registry: Registry):
        self.controller = pequod_controller
        self.core_manager = core_manager
        self.lb_updater = lb_updater
        self.registry = registry

    @classmethod
    def setup_bootstrap(cls, hostname, bootstrap_files):
        core_manager = BootstrapCore(bootstrap_files)
        controller = BootstrapController(hostname, bootstrap_files)
        lb_updater = BootstrapLBUpdater(bootstrap_files)
        registry = BootstrapRegistry(bootstrap_files)
        return cls(controller, core_manager, lb_updater, registry)

    @classmethod
    def setup_core_seeds(cls, hostname, core_seeds):
        # FIXME: remove hardcoded credentials and use initial IP-based authentication to retrieve agent credentials
        credentials = {'username': 'root', 'password': 'root'}
        controller = ClusterController(hostname, core_seeds, credentials)
        core_manager = ClusterCore(core_seeds, credentials)
        registry = ClusterRegistry(core_seeds, credentials)
        lb_updater = ClusterLBUpdater(controller, registry, credentials)
        return cls(controller, core_manager, lb_updater, registry)