#!/usr/bin/env python3
import logging

from pequod_agent.models.cluster_state import ClusterState

class Controller:
    """
    Base object for controller objects
    """

    def poll(self) -> ClusterState:
        raise NotImplementedError

    def register(self, node_host: str, node_state: dict):
        raise NotImplementedError

    def update_instance(self, instance):
        raise NotImplementedError

    def delete_instance(self, instance_name: str):
        raise NotImplementedError


class LocalController(Controller):
    """
    Base class for local testing controllers
    """
    def __init__(self):
        self.cluster_state = dict()

    def register(self, node_host: str, node_state: dict):
        logging.info('Registration: %s', node_state)
        self.cluster_state[node_state.host] = node_state

    def update_instance(self, instance):
        logging.info('Upd instance: %s', instance)


