#!/usr/bin/env python3

import xml.etree.ElementTree as ET


class Manifest:
    """
    Application Manifest: "static" definition of an application+version
    """

    def __init__(self, data):
        self.data = data

    def __str__(self):
        return str(self.data)  # pragma: no cover

    @property
    def resources(self):
        return self.data.get('resources', {})

    @property
    def filesystems(self):
        return self.data.get('filesystems', [])

    @property
    def services(self):
        return self.data.get('services', [])

    @property
    def network_rules(self):
        return self.data.get('network_rules', [])

    @property
    def vaults(self):
        return self.data.get('vaults', [])

ATTRIBUTE_TYPES = {
    'default-mb': int,
    'min-mb': int,
    'local-port': int,
}

def convert_attributes(d: dict, default_type=str) -> dict:
    return {key.replace('-', '_'): ATTRIBUTE_TYPES.get(key, default_type)(val) for key, val in d.items()}

def read_manifest_xml(xml: str) -> Manifest:
    NS = 'http://tech.zalando.com/pequod/schema/registry'

    root = ET.fromstring(xml)

    data = {}

    resources = {}
    for tag in root.find('{' + NS + '}resources') or []:
        row = {}
        row.update(convert_attributes(tag.attrib, default_type=int))
        resources[tag.tag.split('}')[1]] = row
    data['resources'] = resources

    for section in 'filesystems', 'services', 'network-rules', 'vaults':
        entries = []
        for tag in root.find('{' + NS + '}' + section) or []:
            row = {'type': tag.tag.split('}')[1]}
            row.update(convert_attributes(tag.attrib))
            entries.append(row)
        data[section.replace('-', '_')] = entries
    return Manifest(data)



