#!/usr/bin/env python3

import yaml
from pequod_agent.constants.path import PERSISTENCE_PATH
from uuid import uuid4

# TODO: Collection object

def _get_or_set_uuid(obj):
    try:
        uuid = obj._persistence_uuid
    except AttributeError:
        uuid = uuid4().hex
        obj._persistence_uuid = uuid
    return uuid

def save(obj, collection:str):
    # save object on Persistence Path
    uuid = _get_or_set_uuid(obj)
    # ypo - yaml persisted object
    filename = '{collection}-{uuid}.ypo'.format(**locals())
    path = PERSISTENCE_PATH / filename
    yaml.dump(obj, stream= path.open('w+'))

def remove(obj, collection:str):
    # save object on Persistence Path
    uuid = _get_or_set_uuid(obj)
    # ypo - yaml persisted object
    filename = '{collection}-{uuid}.ypo'.format(**locals())
    path = PERSISTENCE_PATH / filename
    try:
        path.unlink()
    except FileNotFoundError:
        # File was already removed
        pass

def load_all(collection:str):
    glob = '{collection}-*.ypo'.format(**locals())
    return [yaml.load(path.open()) for path in PERSISTENCE_PATH.glob(glob)]
